require('babel/register');

var express = require('express')
var shortid = require('shortid')
var events  = require('events')
var fixture = require('./src/fixture')
var faker = require('faker')
var notifier = require('node-notifier')
var _ = require('lodash')

var moment = require('moment')

function createBridge (options) {
  var _records = []
  var _events  = new events.EventEmitter()

  return {
    on: _events.on.bind(_events),

    emit: _events.emit.bind(_events),

    all: function () {
      return _records;
    },

    find: function (id) {
      for (var i = 0; i < _records.length; i++) {
        if (_records[i].id === id) {
          return _records[i]
        }
      }

      return false
    },

    clear: function () {
      _records = []
    },

    record: function (req) {
      var id  = shortid.generate()
      var record = {
        id: id,
        status: 'open',
        method: req.method,
        path: req.path,
        request: req,
        started_at: moment(),
        ended_at: null,
        meta: [],
        tags: []
      }

      _records.push(record)
      _events.emit('request-open', record)

      return record
    },

    close: function (record) {
      record.status = 'closed'
      record.ended_at = moment()
      _events.emit('request-close', record)
    }
  }
}

function captureRequest (bridge) {
  return function (req, res, next) {
    var chunks = []
      , oldWrite = res.write
      , oldEnd = res.end
      , record = bridge.record(req)

    // add the record to the request
    req.record = record

    // add the bridge to the request
    req.restfool = bridge

    // replace the write() method
    res.write = function (chunk) {
      chunks.push(chunk)
      oldWrite.apply(res, arguments)
    }

    // replace the sendStatus() method

    // replace the end() method
    res.end = function (chunk) {
      if (chunk) chunks.push(chunk)
      var body = Buffer.concat(chunks).toString('utf8')
      record.response = res
      bridge.close(record)
      oldEnd.apply(res, arguments)
    }

    next()
  }
}

var createDashboard = require('./src/dashboard/routes')


var userFixture = fixture({
  name: function () {
    return faker.name.firstName() + ' '+ faker.name.lastName()
  },
  email: faker.internet.email
})




var app = express()
var bridge = createBridge()

bridge.on('request-close', function (record) {
  notifier.notify({
    title: 'RESTfool ~ Sent ' + record.response.statusCode,
    message: record.method + ' ' + record.path
  })
})


app.use(captureRequest(bridge))

app.get('/', function (req, res, next) {
  res.send('ok....')
})

app.use(function (req, res, next) {
  res.status(404).end()
})

app.listen(4500)

var dash = express()
dash.use(createDashboard(bridge))
dash.listen(4501)
